/*
Chris Busch
*/

//This class was intended to make graphics implementation independent...
/*naming convention note:
        name <=> unname
        putname <=> getname //for changing 'current' screen
        setname <=> getname //setting internal var for future updates
*/

#ifndef SCREENIO_H
#define SCREENIO_H

#ifdef __linux__
   #include <ncurses/ncurses.h>             
#else
  #define NOMACROS
  #include <curses.h>
#endif


//Will need to get rid of annoying macros...
//since unfortunately macros can have strange
//side-effects eg curses has clear() macro-ed!!

#ifndef NOMACROS
#ifdef clear
           #undef clear
           inline int clear() { return wclear(stdscr); }
#endif
//because of:
//#define       move(y, x)      wmove(stdscr, y, x)
//#define       refresh()       wrefresh(stdscr)
#ifdef move
           #undef move
           inline int move(int y,int x) {return wmove(stdscr,y,x);}
#endif
#ifdef refresh
           #undef refresh
           inline int refresh() {return wrefresh(stdscr);}
#endif

#endif //nomacros
                                           
typedef int sioret;
typedef int pixtype;
typedef WINDOW* asioptr;


//these HACK functions are for accessing curses stuff
inline sioret HACKbeep()                { return beep();}
inline sioret HACKflash()               { return flash();}
inline sioret HACKrefresh()             { return refresh();}


class screenio
{
protected:
		friend void screenio_handler(int);
        pixtype bg;
        int active;
        int keyactive;

		void (*cursessigcont)(int);

		volatile int haschanged;
public:
//		enum {nomode,textmode,cursesmode};

        screenio();
        ~screenio() { }

		volatile int changed() 
		{ 
		  return haschanged;
		}        

		void clearchanged() { haschanged=0; }
		
        int isactive() { return active; }
        int kbactive() { return keyactive; }

        int xsize() { return COLS; }
        int ysize() { return LINES;   }

        asioptr init();

        bool uninit();

        pixtype setbg(pixtype b)        { return bg=b;}
        pixtype getbg()         {return bg;}

        sioret beep()           { return HACKbeep();}
        sioret flash()          { return HACKflash();}

        sioret suspend()        {return endwin();}      
        sioret unsuspend()      {return refresh();}

        sioret refresh()        {return HACKrefresh();}
        sioret repaint()        {return wrefresh(curscr);}
        sioret cls()            {return clear(); }

        sioret puttext(int x,int y,char s[])
        { return mvaddstr(y,x,s); }

        sioret putpixel(int x,int y,pixtype c)
        { return mvaddch(y,x,c); }
        pixtype getpixel(int ,int )  //int x int y
        { return bg; }

//      getsyx(y, x)            _getsyx(&(y), &(x))
//#define       getyx(win, y, x)        ((y) = getcury(win), (x) = getcurx(win))

        sioret initkey();
        sioret uninitkey();
        sioret getkey() { return getch(); }  //mmm... wonder if correct for C++
                
//		int setmode(int);
};

extern class screenio sio;

#endif //this file

